//=====================================================================================================================
/// @file		I2C_atmega328p.c
///
/// @brief		Variables and functions for I2C (aka TWI) Interface of atmega328p
///
/// @version	V1.0
///
/// @date		15.03.21
///
/// @author		Philipp Holle
//=====================================================================================================================

//---------------------------------------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------------------------------------
#include "I2C_atmega328p.h"
//---------------------------------------------------------------------------------------------------------------------
// Global variables
//---------------------------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------------------------
// Global functions
//---------------------------------------------------------------------------------------------------------------------
//=== Function: I2cInit		  =============================================================================================
///
/// @brief	Initialize I2C Interface (aka TWI)
///
/// @param uint16_t bitRate //in kHz; Max: MCU Freq / 16, Min: MCU Freq / 526
///
/// @return
//=====================================================================================================================
void I2cInit(uint8_t sysClock, uint16_t bitRate)
{
	//Set Slave Adress
	TWAR = ((I2C_ADDRESS<<1) & 0xFE);
	//Set bit rate
	TWBR = (((sysClock*1000)/(2*bitRate))-8);
}
//=== Function: I2cTxMaster	=============================================================================================
///
/// @brief	Send Data via I2C as Master
///
/// @param	
///
/// @return
//=====================================================================================================================
bool i2cTxMaster (uint8_t *firstelementofData,uint8_t lengthOfData,uint8_t address)
{
	//aux variable
	uint8_t a;
	//Result variable 
	bool result = true;
	//Loop Variable
	uint8_t i=0;
	//Transmit START condition
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	
	a= TWCR;
	UartPrintLine("IC2 send START ",&a,1);
	UartPrintNewLine();
	
	//Wait for TWINT flag
	while(!(TWCR & (1<<TWINT)));
	
	a= TWSR;
	UartPrintLine("TWSR: ",&a,1);
	UartPrintNewLine();
	
	//Terminate if status != 0x08
	if ((TWSR & 0xF8) != 0x08)
	{
		UartPrintLine("Error: ",0, 0);
		UartPrintNewLine();
		
		
		result = false;
		return result;
		
		
		
	}
	UartPrintLine("I2C START ok",0,0);
	UartPrintNewLine();
	//Load SLA+W to data register
	TWDR = ((address<<1) & 0xFE);
	
	a = TWDR;
	UartPrintLine("Transmit address+w ",&a,1);
	UartPrintNewLine();
	
	//Transmit Address Byte
	TWCR = (1<<TWINT)|(1<<TWEN);
	//Wait for TWINT flag
	while(!(TWCR & (1<<TWINT)));
	//Terminate if status != 0x18
	if ((TWSR & 0xF8) != 0x18)
	{
		UartPrintLine("Error: ",0, 0);
		UartPrintNewLine();
		result = false;
		return result;
	}
	
	UartPrintLine("Address acknowledged",0,0);
	UartPrintNewLine();
	
	//Transmit data bytes
	for (i=0; i<lengthOfData; i++)
	{
		//Load current byte to data register
		TWDR = *(firstelementofData+i);
		//Send data byte
		TWCR = (1<<TWINT)|(1<<TWEN);
		//Wait for TWINT flag
		while(!(TWCR & (1<<TWINT)));
		//Terminate if status != 0x28
		if ((TWSR & 0xF8) != 0x28)
		{
			result = false;
			return result;
		}
	}
	
	UartPrintLine("Data transmission successful",0,0);
	UartPrintNewLine();
	
	//Transmit STOP Condition
	TWCR = (1<<TWINT)|(1<<TWSTO)|(1<<TWEN);
	return result;
}
/*
ISR(TWI_vect)
{
	//Disable TWI Interrupt
	TWCR &= !(1<<TWIE);
	if (((TWSR & 0xF8) == 0x60) || ((TWSR & 0xF8) == 0x70))
	{
		payloadPendingUpLength=0;
		//Clear TWINT flag
		TWCR |= (1<<TWINT);
		//Wait for TWINT flag
		while(!(TWCR & (1<<TWINT)));
		while ((TWSR & 0xF8) != 0xA0)
		{
			payloadPendingUpContent[payloadPendingUpLength] = TWDR;
			payloadPendingUpLength++;
			//Clear TWINT flag
			TWCR |= (1<<TWINT);
			//Wait for TWINT flag
			while(!(TWCR & (1<<TWINT)));
		}
	}
	payloadPendingUp = true;
	//Enable TWI Interrupt
	TWCR |= (1<<TWIE);
}
*/